import rclpy
from rclpy.node import Node

import add_on_msgs.srv

rclpy.init()
node = Node("test_srv")

# clients
srv_get_prims = node.create_client(add_on_msgs.srv.GetPrims, '/get_prims')
srv_get_attributes = node.create_client(add_on_msgs.srv.GetPrimAttributes, '/get_attributes')
srv_get_attribute = node.create_client(add_on_msgs.srv.GetPrimAttribute, '/get_attribute')
srv_set_attribute = node.create_client(add_on_msgs.srv.SetPrimAttribute, '/set_attribute')

# requests
request_get_prims = add_on_msgs.srv.GetPrims.Request()
request_get_attributes = add_on_msgs.srv.GetPrimAttributes.Request()
request_get_attribute = add_on_msgs.srv.GetPrimAttribute.Request()
request_set_attribute = add_on_msgs.srv.SetPrimAttribute.Request()

# wait for services to become available
print("Waiting for /get_prims")
srv_get_prims.wait_for_service(timeout_sec=None)
print("Waiting for /get_attributes")
srv_get_attributes.wait_for_service(timeout_sec=None)
print("Waiting for /get_attribute")
srv_get_attribute.wait_for_service(timeout_sec=None)
print("Waiting for /set_attribute")
srv_set_attribute.wait_for_service(timeout_sec=None)


request_get_prims.path = "/"
future = srv_get_prims.call_async(request_get_prims)
rclpy.spin_until_future_complete(node, future)
prims = future.result()

print("Prims")
for prim_path, type_name in zip(prims.paths, prims.types):
    print("  |-- {} ({})".format(prim_path, type_name))

    # attributes
    request_get_attributes.path = prim_path
    future = srv_get_attributes.call_async(request_get_attributes)
    rclpy.spin_until_future_complete(node, future)
    attributes = future.result()

    for attribute_name, attribute_display, attribute_type in zip(attributes.names, attributes.displays, attributes.types):
        print("  |     |-- {} ({})".format(attribute_name, attribute_type))

        # attribute
        request_get_attribute.path = prim_path
        request_get_attribute.attribute = attribute_name

        future = srv_get_attribute.call_async(request_get_attribute)
        rclpy.spin_until_future_complete(node, future)
        attribute = future.result()

        print("  |     |     |-- {} ({})".format(attribute.value if len(attribute.value) < 50 else attribute.value[:50] + "...", attribute.type))

        request_set_attribute.path = prim_path
        request_set_attribute.attribute = attribute_name
        request_set_attribute.value = attribute.value

        future = srv_set_attribute.call_async(request_set_attribute)
        rclpy.spin_until_future_complete(node, future)
        response = future.result()

        if not response.success:
            print("[ERROR]", response.message)
        print(response)
